#ifndef ___iso9660___                                      
#define ___iso9660___

#pragma member_alignment __save
#pragma nomember_alignment byte

/*
 ******************************************************************************
 *
 * iso9660.h
 *
 ******************************************************************************
 */

/*
 * Global constants
 */
#define CDR_SECTOR_SIZE         2048
#define CDR_DATA_SECTOR         16
#define CDR_ISO_VERSION         1
#define CDR_DIRID_ROOT          0
#define CDR_DIRID_PARENT        1

typedef struct
{
    unsigned int            littleEndianValue;
    unsigned int            bigEndianValue;
} BothEndDWord;

typedef struct
{
    unsigned short int      littleEndianValue;
    unsigned short int      bigEndianValue;
} BothEndWord;

typedef struct
{
    char            year [4];               /* ASCII year                           */
    char            month [2];              /* ASCII month (01 = January, etc.)     */
    char            day [2];                /* ASCII day of month (01..31)          */
    char            hour [2];               /* ASCII hour (00..23)                  */
    char            minute [2];             /* ASCII minute (00..59)                */
    char            second [2];             /* ASCII second (00..59)                */
    char            centiSecond [2];        /* ASCII hundredths of a second (00..99)*/
    signed char     gmtOffset;              /* Offset from GMT in 15-minute units   */
} AscDateTime;

typedef struct
{
    unsigned char   year;                   /* Number of years since 1900           */
    unsigned char   month;                  /* Month (1..12)                        */
    unsigned char   day;                    /* Day of month (1..31)                 */
    unsigned char   hour;                   /* Hour of day (0..23)                  */
    unsigned char   minute;                 /* Minute of hour (0..59)               */
    unsigned char   second;                 /* Second of minute (0..59)             */
    signed char     gmtOffset;              /* Offset from GMT in 15-minute units   */
} BinDateTime;

/*
 * Volume Descriptor types
 */
#define CDR_VD_BOOT     0                   /* Boot Record Descriptor               */
#define CDR_VD_PVD      1                   /* Primary Volume Descriptor            */
#define CDR_VD_SVD      2                   /* Supplementary Volume Descriptor      */
#define CDR_VD_VPD      3                   /* Volume Partition Descriptor          */
#define CDR_VD_VDST     255                 /* Volume Descriptor Set Terminator     */

typedef struct
{
    unsigned char   type;                   /* Volume descriptor type (1)           */
    char            identifier [5];         /* Standard identifier (CD001)          */
    unsigned char   version;                /* Volume descriptor version (1)        */
    char            unused1;                /* Unused field                         */
    char            systemId [32];          /* System identifier                    */
    char            volumeId [32];          /* Volume identifier                    */
    char            unused2 [8];            /* Unused field                         */
    BothEndDWord    sectorCount;            /* Volume sector count                  */
    char            unused3 [32];           /* Unused field                         */
    BothEndWord     volumeSetCount;         /* Volume set count                     */
    BothEndWord     volumeSeqNumber;        /* Volume sequence number               */
    BothEndWord     sectorSize;             /* Sector size (2048)                   */
    BothEndDWord    pathTableSize;          /* Path table size (in bytes)           */
    unsigned int    pathTableSectorLE;      /* Path table starting sector number    */
                                            /*  (in little endian format)           */
    unsigned int    oPathTableSectorLE;     /* Optional pathTableSector             */
                                            /*  (in little endian format)           */
    unsigned int    pathTableSectorBE;      /* Path table starting sector number    */
                                            /*  (in big endian format)              */
    unsigned int    oPathTableSectorBE;     /* Optional pathTableSector             */
                                            /*  (in big endian format)              */
    char            rootDirectory [34];     /* Root directory record                */
    char            volumeSetIdent [128];   /* Volume set identification            */
    char            publisherIdent [128];   /* Publisher identification             */
    char            preparerIdent [128];    /* Preparer identification              */
    char            applicationIdent [128]; /* Application identification           */
    char            copyrightFileId [37];   /* Copyright file identifier            */
    char            abstractFileId [37];    /* Abstract file identifier             */
    char            biblioFileId [37];      /* Bibliographic file identifier        */
    AscDateTime     creationDate;           /* Volume creation date/time            */
    AscDateTime     modificationDate;       /* Volume modification date/time        */
    AscDateTime     expirationDate;         /* Volume expiration date/time          */
    AscDateTime     effectiveDate;          /* Volume effective date/time           */
    unsigned char   structVersion;          /* File structure version (1)           */
    unsigned char   unused4;                /* Unused field                         */
    unsigned char   applicationData [512];  /* Application data field               */
    unsigned        unused5 [653];          /* Unused field                         */
} PrimaryVolumeDescriptor;

typedef struct
{
    unsigned char   type;                   /* Volume descriptor type (255)         */
    char            identifier [5];         /* Standard identifier (CD001)          */
    unsigned char   version;                /* Volume descriptor version (1)        */
    unsigned char   zeros [2041];           /* Zero fill                            */
} VolumeDescriptorSetTerminator;

typedef struct
{
    unsigned char   hidden          : 1;    /* File is a hidden file                */
    unsigned char   directory       : 1;    /* File is a directory file             */
    unsigned char   associated      : 1;    /* File is an associated file           */
    unsigned char   recordFormat    : 1;    /* File has record format specified     */
    unsigned char   permissions     : 1;    /* File has permissions specified       */
    unsigned char   unused          : 2;    /* Unused flag                          */
    unsigned char   moreRecords     : 1;    /* Set if not final record for file     */
} FileFlags;

typedef struct
{
    unsigned char   recordLength;           /* Length of record (in bytes)          */
    unsigned char   extendedSectorCount;    /* Number of sectors in attribute record*/
    BothEndDWord    sectorAddress;          /* Starting sector of file/directory    */
    BothEndDWord    fileLength;             /* Length of file/directory (in bytes)  */
    BinDateTime     creationDate;           /* File creation date                   */
    FileFlags       flags;                  /* Various flags                        */
    unsigned char   fileUnitSize;           /* File unit size for interleaved file  */
    unsigned char   fileGapSize;            /* File gap size for interleaved file   */
    BothEndWord     volumeSeqNumber;        /* Volume sequence number (1)           */
    unsigned char   nameLength;             /* Length of file name (in bytes)       */
    char            name [1];               /* File name                            */
/*  char            padding;                /* Optional padding byte                */
} DirectoryRecord;

typedef struct
{
    unsigned char   nameLength;             /* Length of name field (in bytes)      */
    unsigned char   extendedSectorCount;    /* Number of sectors in attribute record*/
    int             sectorAddress;          /* Sector address of directory          */
    short int       parentId;               /* Parent directory record number       */
    char            name [1];               /* Directory name                       */
/*  char            padding;                /* Optional padding byte                */
} PathTableEntry;

#pragma member_alignment __restore
#endif
