
/******************************************************************************
 *
 * Sort.C
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Headers
 *
 ******************************************************************************/

#include <stdio.h>

#include "intrinsic.h"
#include "sort.h"

/******************************************************************************
 *
 * Local Constants and Macros
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Local Declarations
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Local Prototypes
 *
 ******************************************************************************/

#ifdef VMS
static int local_strcmp  (String key, String *array, int index);
static int local_sortcmp (String *array, int index1, int index2);
static int local_swap    (String *array, int index1, int index2);
#else
static int local_strcmp  ();
static int local_sortcmp ();
static int local_swap    ();
#endif

/******************************************************************************
 *
 * SearchBinary
 *
 ******************************************************************************/

#ifdef VMS
extern int SearchBinary (Pointer array, int size, String key, IPFN fcn)
#else
extern int SearchBinary (array, size, key, fcn)

  Pointer array;
  int     size;
  String  key;
  IPFN    fcn;
#endif

{
  int    i,j,k,low,high;

  /* check size first */
  if (size < 1) return(-1);

  /* default the compare function */
  if (!fcn) fcn = local_strcmp;

  low  = 0;  
  high = size - 1;

  do {
    i = (low + high) / 2;
    j = (*fcn)(key,array,i);
    if (!j) break;
    else if (j < 0) high = i - 1;
    else low = i + 1;
  } while (low <= high);

  if (low <= high) return(i);
  else             return(-1);
}

/******************************************************************************
 *
 * Sort
 *
 ******************************************************************************/

#ifdef VMS
extern void Sort (Pointer array, int size, IPFN cmp, IPFN swap)
#else
extern void Sort (array, size, cmp, swap)

  Pointer array;
  int     size;
  IPFN    cmp;
  IPFN    swap;
#endif

{
  int result;
  int index1, index2;

  /* default function */
  if (cmp == NULL)  cmp  = local_sortcmp;
  if (swap == NULL) swap = local_swap;

  /* sort */
  for (index1 = 0; index1 < (size - 1); index1++) {
    for (index2 = index1 + 1; index2 < size; index2++) {
      /* compare */
      result = (*cmp)(array,index1,index2);

      /* swap if needed */
      if (result > 0) (*swap)(array,index1,index2);
    }
  }

  /* done */
  return;
}

/******************************************************************************
 *
 * local_strcmp
 *
 ******************************************************************************/

#ifdef VMS
static int local_strcmp (String key, String *array, int index)
#else
static int local_strcmp (key, array, index)

  String  key;
  String *array;
  int     index;
#endif

{
  /* return the result */
  return (strcmp(key,array[index]));
}

/******************************************************************************
 *
 * local_sortcmp
 *
 ******************************************************************************/

#ifdef VMS
static int local_sortcmp (String *array, int index1, int index2)
#else
static int local_sortcmp (array, index1, index2)

  String *array;
  int     index1;
  int     index2;
#endif

{
  /* return the result */
  return (strcmp(array[index1],array[index2]));
}

/******************************************************************************
 *
 * local_swap
 *
 ******************************************************************************/

#ifdef VMS
static int local_swap (String *array, int index1, int index2)
#else
static int local_swap (array, index1, index2)

  String *array;
  int     index1;
  int     index2;
#endif

{
  String temp;

  /* swap */
  temp          = array[index1];
  array[index1] = array[index2];
  array[index2] = temp;

  /* done */
  return (1);
}
